// 微站上的百度地图

// NOTE!!! 这里的 ./ 一定不要忽略;否则被外部模块加载时,路径解析错误
define(['./trackreplay', './baiduMapApi', './baiduCurveLineApi'],
    function(module){
        return module.directive('icBaiduTrack', function(baiduMapApi, baiduCurveLineApi, $http,$timeout){

            // BD-09坐标系转换
            function fixBD09(paths, fixLonKM, fixLatKM, cb){
                // 1KM大约是9/1000度
                var lonFix = Number(fixLonKM) * 9 / 1000;
                var latFix = Number(fixLatKM) * 9 / 1000;

                var ptFix = null; // 当前修正点
                var ptsNeedFix = []; // 需要修正的点集合
                var ptsNeedFixGroup = []; // 百度修正服务,每次只能接受100个点

                if(paths){
                    // 计算需要多少个修正点,存放在ptsNeedFix中
                    for(var i=0;i<paths.length;i++){
                            var pt = paths[i];
                            // 去除前置的东经E,北纬N;如果以后需要处理西半球或南半球的坐标,这里需要添加符号
                            pt.lon2 = pt.lng;
                            pt.lat2 = pt.lat;
                            if(i===0){ // 第一个点必修正
                                ptFix = pt;
                                ptsNeedFix.push({i:i, lon: pt.lon2, lat:pt.lat2});
                                ptsNeedFixGroup.push({pts: ptsNeedFix, done: false});
                            }
                            else{ // 如果和ptFix的偏移量超出lonFix或latFix,则增加一个修正点
                                if(pt.lon2 - ptFix.lon2 > lonFix || pt.lat2 - ptFix.lat2 > latFix){
                                    ptFix = pt;
                                    ptsNeedFix.push({i:i, lon: pt.lon2, lat:pt.lat2});
                                    // 分组,每100个分一组
                                    if(ptsNeedFix.length === 1){
                                        ptsNeedFixGroup.push({pts: ptsNeedFix, done: false});
                                    }
                                    else if(ptsNeedFix.length === 100){
                                        ptsNeedFix = [];
                                    }
                                }
                            }
                    }

                    // 发往百度服务器计算修正值
                    for(var g=0;g<ptsNeedFixGroup.length;g++){
                        (function(i){
                            baiduMapApi.geoConv(ptsNeedFixGroup[i].pts, function(data){
                                var fixGroup = ptsNeedFixGroup[i];
                                if(data.status == 0 && data.result && data.result.length === fixGroup.pts.length){
                                    for(var k=0;k<data.result.length;k++){
                                        fixGroup.pts[k].deltaLon = data.result[k].x - fixGroup.pts[k].lon;
                                        fixGroup.pts[k].deltaLat = data.result[k].y - fixGroup.pts[k].lat;
                                    }
                                }
                                fixGroup.done = true;

                                // 全部DELTA值计算完成
                                for(var j=0;j<ptsNeedFixGroup.length;j++) {
                                    if (ptsNeedFixGroup[j].done === false) return;
                                }

                                // 拼接全部GROUP统一处理
                                var m,n;
                                var ptsFixAll = [];
                                for(m=0;m<ptsNeedFixGroup.length;m++){
                                    for(n=0;n<ptsNeedFixGroup[m].pts.length;n++){
                                        ptsFixAll.push(ptsNeedFixGroup[m].pts[n]);
                                    }
                                }

                                var fixedCurrent = null, fixedNext = null, idxFixedNext = 1;
                                if(ptsFixAll.length > 0) fixedCurrent = ptsFixAll[0];
                                if(ptsFixAll.length > idxFixedNext) fixedNext = ptsFixAll[idxFixedNext];
                                console.log("共计修正点数:" + ptsFixAll.length);

                                if(fixedCurrent) { // 至少有一个修正点
                                    for (m = 0; m < paths.length; m++) {
                                            var pt = paths[m];
                                            if (fixedNext === null || m < fixedNext.i || ( m === fixedNext.i )) {
                                                // 使用fixedCurrent进行修正
                                                pt.lon2 = Number(pt.lon2) + fixedCurrent.deltaLon;
                                                pt.lat2 = Number(pt.lat2) + fixedCurrent.deltaLat;
                                            }
                                            else{
                                                // 使用下一个修正点
                                                fixedCurrent = fixedNext;
                                                pt.lon2 = Number(pt.lon2) + fixedCurrent.deltaLon;
                                                pt.lat2 = Number(pt.lat2) + fixedCurrent.deltaLat;
                                                // 修正点后移一个
                                                idxFixedNext++;
                                                // console.log("fix point next" + JSON.stringify(fixedCurrent));
                                                if(idxFixedNext < ptsFixAll.length){
                                                    fixedNext = ptsFixAll[idxFixedNext];
                                                }
                                                else{
                                                    fixedNext = null; // 已到最后一个修正点
                                                }
                                            }
                                    }
                                }
                                // 发起回调
                                if(typeof cb === 'function') cb(paths);
                            })
                        })(g);
                    }
                }
            }

            var car; //汽车图标
            var label; //信息标签
            var timer;

            var index = 0;
            var intA = 0;
            var speed = 1;
            var tempPoint = "";
            var points = [];

            function init(map,tracks,cb) {
                //初始化地图,选取第一个点为起始点
                map.addControl(new BMap.NavigationControl());
                map.addControl(new BMap.ScaleControl());
                //map.addControl(new BMap.OverviewMapControl({isOpen: true}));
                //centerPoint = new BMap.Point((tracks[0].lon2 + tracks[tracks.length - 1].lon2) / 2, (tracks[0].lat2 + tracks[tracks.length - 1].lat2) / 2);
                //centerPoint = new BMap.Point(tracks[0].lon2, tracks[0].lat2);
                //map.centerAndZoom(centerPoint,13);

                //显示小车子
                label = new BMap.Label("", {offset: new BMap.Size(-20, -20)});
                var icon = new BMap.Icon("./track-replay/images/caronline.png", new BMap.Size(40,19)); // 设置图片偏移});
                //rotation:pos.direction
                car = new BMap.Marker(new BMap.Point(tracks[0].lon2,tracks[0].lat2),{icon:icon});
                label.setStyle({                                   //给label设置样式，任意的CSS都是可以的
                    color:"black",                   //颜色
                    fontWeight:"bold",
                    fontSize:"14px",               //字号
                    height:"0",                //高度
                    width:"0",                 //宽
                    border:0,
                    textAlign:"center",            //文字水平居中显示
                    lineHeight:"18px"         //行高，文字垂直居中显示
                });
                car.setLabel(label);
                map.addOverlay(car);
                points = [];
                //var tempPoints;
                for(var i=0;i<tracks.length;i++){
                    var point = new BMap.Point(tracks[i].lon2,tracks[i].lat2);
                    points.push(point);
                    //if(!tempPoints)tempPoints = point;
                    //map.addOverlay(new BMap.Polyline([tempPoints,point], {strokeColor: "black", strokeWeight: 4, strokeOpacity: 1}));
                    //tempPoints = point;
                }
                map.setViewport(points);
                map.addOverlay(new BMap.Polyline(points, {strokeColor: "black", strokeWeight: 4, strokeOpacity: 1}));

                cb("drawover");
            }

            function getDirection(tmpPoint,point){
                var x = tmpPoint.lat - point.lat;
                var y = tmpPoint.lng - point.lng;
                var direction = parseInt((Math.atan2(x, y) * 180 / Math.PI) * -1);
                direction = direction - 180;
                return direction;
            }

            function getSpeedColor(speed){
                if( speed > 120){
                    var strokeColor = "#0F253F";
                }else if( speed < 120 && speed >=90){
                    var strokeColor = "#52DC52";
                }else if( speed < 90 && speed >= 45){
                    var strokeColor = "#5E93CE";
                }else if(speed < 45 && speed >= 15){
                    var strokeColor = "#E46D0A";
                }else{
                    var strokeColor = "#A40408";
                }
                return strokeColor;
            }


            //播放
            function play(map,locations,tracks){
                var point =  new BMap.Point(tracks[intA].lon2,tracks[intA].lat2);
                var color = getSpeedColor(tracks[intA].speed);
                if(tempPoint === "") tempPoint = point;
                else{
                    var direction = getDirection(tempPoint,point);
                    car.setRotation(direction);
                }
                map.addOverlay(new BMap.Polyline([tempPoint, point], {strokeColor: color, strokeWeight: 4, strokeOpacity: 1}));
                //console.log(typeof(String(point.lng)));
                label.setContent("经度: " + String(point.lng).substr(0,10) + "<br>纬度: " + String(point.lat).substr(0,10));
                car.setPosition(point);
                $("#tr"+intA).css("background-color","grey");//回放过的记录变成灰色
                intA++;
                tempPoint = point;
                //画面跟随
                //if(followChk.checked) {
                    map.panTo(point);
                //}
                if(intA < tracks.length-1) {
                  var time= tracks[intA].minustime / speed;
                  timer = $timeout(function(){
                         play(map,locations,tracks);
                    },time);
                }else if(intA == tracks.length-1){ //最后一个点直接达到
                    play(map,locations,tracks);
                    map.setViewport(points);
                    playover();
                }
            }

            //播放结束
            function playover(){
                $("#pause").attr("disabled",true);
                $("#reset").removeAttr("disabled");
            }

            //暂停
            function pause() {
                if(timer){
                    $timeout.cancel(timer);
                }
            }


            // Link icBaiduTrack
            function link(scope, element, attrs){
                var map = null;
                //获取所有点的坐标
                // 我们使用异步方式加载百度地图API,很可能此时还没有完成加载过程
                // 因此,当百度地图异步加载完成后,需要通过回调完成初始化
                var locations = [];
                var tracks = [];
                baiduMapApi.initBaiduMap(function(){
                    var id = attrs['id'];
                    if(!id){ // 如果没有id,那么随机分配一个
                        id = "trackreplay_" + Math.random().toString().substr(2, 8);
                        element.attr('id', id);
                        attrs['id'] = id;
                    }
                    map = new BMap.Map(id);
                    map.centerAndZoom(new BMap.Point(106.28501, 35.561577), 5); // 天安门
                    map.enableScrollWheelZoom();
                    // 如果在地图初始完成之前,icPaths数据已经准备好,那么立即绘制一次
                    if(scope.icScale){
                        speed = scope.icScale;
                        console.log(speed);
                    }
                    if(scope.icTracks){
                        fixBD09(scope.icTracks, 20, 20, function (valueFixed) {
                            tracks = valueFixed;
                            init(map,tracks,function(data){
                                $("#play").removeAttr("disabled");
                                $("#pause").attr("disabled");
                                $("#reset").attr("disabled");
                            });
                        });
                    }
                });

                // 监视icTracks
                scope.$watch('icTracks', function(value){
                    // 地图初始后,每次icPaths更新,都需要重新绘制路径
                    if(map){
                         locations = [];
                         index = 0;
                         intA = 0;
                        //登录车队时，点击车辆管理，会出现length错误。
                        if(value== undefined){
                            value=[];
                        }
                        if(value.length>0){
                            fixBD09(value, 20, 20, function (valueFixed){
                                tracks = valueFixed;
                                scope.initMap();
                                init(map,tracks,function(data){
                                    $("#play").removeAttr("disabled");
                                    $("#pause").attr("disabled",true);
                                    $("#reset").attr("disabled",true);
                                });
                            });
                        }else{
                            scope.initMap();
                        }
                    }
                });

                scope.initMap = function(){
                    var id = attrs['id'];
                    if(!id){ // 如果没有id,那么随机分配一个
                        id = "trackreplay_" + Math.random().toString().substr(2, 8);
                        element.attr('id', id);
                        attrs['id'] = id;
                    }
                    map = new BMap.Map(id);
                    map.centerAndZoom(new BMap.Point(106.28501, 35.561577), 5); // 天安门
                    map.enableScrollWheelZoom();
                }

                //监听播放
                scope.$watch('icPlay',function(value){
                    if(value) {
                        $("#play").attr("disabled", true);
                        $("#pause").removeAttr("disabled");
                        $("#reset").attr("disabled", true);
                        play(map, locations, tracks);
                    }
                });

                //监听暂停
                scope.$watch('icPause',function(value){
                    if(value){
                        $("#play").removeAttr("disabled");
                        $("#pause").attr("disabled",true);
                        $("#reset").attr("disabled",true);
                        pause();
                    }
                });

                //监听重置
                scope.$watch('icReset',function(value){
                    if(value){
                        intA = 0;
                        tempPoint = "";
                        for(var i=0;i<tracks.length;i++){
                            $("#tr"+i).css("background-color","");//重置变成白色背景
                        }
                        scope.initMap();
                        init(map,tracks,function(data){
                            $("#play").removeAttr("disabled");
                            $("#pause").attr("disabled",true);
                            $("#reset").attr("disabled",true);
                        });
                    }
                });

                //监听选择的速度
                scope.$watch('icScale',function(value){
                    if(value){
                        speed = value;
                    }
                });
            }
            
            return {
                restrict:'A',
                scope: {
                    icBaiduTrack: '=', // 地图的初始位置
                    icTracks: '=', // 地图上的路径数据
                    icPlay: '=', //播放按钮
                    icPause: '=', //暂停
                    icReset: '=', //重置
                    icScale: '=' //加倍速度
                },
                link: link
            };
        });
});